@@ -14,11 +14,19 @@
 #define CD_FRAMESIZE_RAW 2352
 #endif
 
+/* this includes 2352 data bytes + 96 subchannel bytes (P-W) */
 #undef CD_FRAMESIZE_RAWER
 #define CD_FRAMESIZE_RAWER 2448
 
+/* this includes 2352 data bytes
+                + 296 c2 and block error flag bytes
+                +  96 subchannel bytes (P-W) */
+#undef CD_FRAMESIZE_RAWEST
+#define CD_FRAMESIZE_RAWEST 2744
+
 #define CD_FRAMESAMPLES (CD_FRAMESIZE_RAW / 4)
 #define CD_FRAMEQBYTES (CD_FRAMESIZE_RAWER - CD_FRAMESIZE_RAW)
+#define CD_FRAMEEBYTES (CD_FRAMESIZE_RAWEST - CD_FRAMESIZE_RAW - CD_FRAMEQBYTES)
 
 #define MAXTRK	100	/* maximum of audio tracks */
 
@@ -38,6 +46,7 @@ extern unsigned char MCN[15];
 
 extern unsigned interface;
 extern unsigned char *bufferCdda;
+extern unsigned cd_framesize;
 
 #define NSECTORS 75
 
@@ -46,7 +55,7 @@ extern unsigned char *bufferCdda;
 
 /* cdrom access function pointer */
 extern unsigned (*ReadToc) ( TOC *ptoc );
-extern void     (*ReadCdRom) (unsigned char *p, long lSector, unsigned long SectorBurstVal );
+extern void     (*ReadCdRom) (unsigned char *p, long lSector, unsigned long SectorBurstVal, int with_errorblockflags );
 extern void     (*SelectSpeed) ( unsigned speed );
 
-void SetupInterface( void );
+void SetupInterface( unsigned framesize );
