@@ -23,7 +23,7 @@
 #include <assert.h>
 
 #include "mycdrom.h"
-#include "readall.h"
+#include "read2448.h"
 #include "interface.h"
 #include "byteorder.h"
 #include "ringbuff.h"
@@ -40,26 +40,10 @@ SaveBuffer (unsigned char *p,
 	    unsigned long SamplesToDo, 
 	    unsigned long *TotSamplesDone)
 {
-  char *pSrc;                   /* start of cdrom buffer */
-  char *pSrcStop;               /* end of cdrom buffer */
-
-  pSrc = ( char * ) p;
-  pDst = ( unsigned char * ) pSrc;
-  pStart = ( unsigned char * ) pSrc;
-  pSrcStop = pSrc + SamplesToDo*4;
-
-  /* code for subsampling and output stage */
-
-  /* optimize the case of no conversion */
-  if ( 1 ) {
-    /* output format is the original cdda format ->
-     * just forward the buffer 
-     */
-      
-    pDst = (unsigned char *) pSrcStop;		/* set pDst to end */
-  }
-  global.SkippedSamples += (pSrcStop - (char *)p)/4;
+  pDst = p + SamplesToDo*4;
+  pStart = p;
 
+  /* code for output stage */
   if ( 1) {
     int retval = 0;
     unsigned outlen;
