@@ -5,4 +5,4 @@
 #include <stddef.h>
 #include "mytype.h"
 
-#define entry_size (global.nsectors*CD_FRAMESIZE_RAWER)
+#define entry_size (global.nsectors*cd_framesize)
