@@ -454,42 +454,15 @@ if (DEBUG_HEUR) fprintf(stdout, "zero = %x\n", u2.q.zero);
   return result;
 }
 
-static unsigned char sectbuff[CD_FRAMESIZE_RAW + SUBSIZE];
-
-int main(int argc, char **argv)
+void x(unsigned char *subs, unsigned *sectors_trackrel) 
 {
-  FILE *fp;
   int i;
-  unsigned char *subs;
   unsigned last_asec = 0xffffffffU;
   unsigned last_rsec = 0xffffffffU;
-  unsigned count_sectors = 0;
-  unsigned sectors_trackrel = 0;
   unsigned char tr=1, in=0;
-  int with_data = 1;
   unsigned count_rotten_entries = 0;
-
-  if (argc == 3 && !strcmp(argv[1], "-s")) {
-    argv++; argc--;
-    with_data = 0;
-  }
-  if (argc != 2) return 1;
-
-  fp = fopen(argv[1], "rb");
-  if (fp == NULL) return 2;
-
-  for (;! feof(fp); count_sectors++, sectors_trackrel++ ) {
     int crcres = 0;
 
-    if (with_data)
-    {
-      fread(sectbuff, CD_FRAMESIZE_RAW + SUBSIZE, 1, fp);
-      subs = sectbuff + CD_FRAMESIZE_RAW;
-    } else {
-      fread(sectbuff, SUBSIZE, 1, fp);
-      subs = sectbuff;
-    }
-
     crcres = getQ(subs);
 
     if (0 != crcres) count_rotten_entries++;
@@ -502,7 +475,7 @@ int main(int argc, char **argv)
       } else {
         last_rsec++;
       }
-      continue;
+      return;
     }
 
 #if 1
@@ -558,7 +531,169 @@ int main(int argc, char **argv)
         in = u.q.index;
       }
     }
+}
+
+/* sub channels use their own error detection/correction data.
+
+   1. P
+      (24,20) Reed-Solomon code over Galoisfield(2^6)
+      6 bit symbols, 4 parity bits
+      P(x) = x^6 + x + 1
+      primitive element: a = 000010
+
+
+   2. Q
+      (4,2) Reed-Solomon code over Galoisfield(2^6)
+      6 bit symbols, 2 parity bits
+      P(x) = x^6 + x + 1
+      primitive element: a = 000010
+
+ */
+
+#include <string.h>
+#define BITS 6
+#define HIGH_LIM (1 << BITS)
+#define HIGH_VAL (HIGH_LIM - 1)
+#define HIGH_BIT (1 << (BITS-1))
+#define INIT_VALUE 2
+#define POL 0x03	/* x^6 + x + 1 */
+
+typedef struct {
+  unsigned char  mode_item;
+  unsigned char  instruction;
+  unsigned char  parityQ[2];
+  unsigned char  data[16];
+  unsigned char  parityP[4];
+} Subcode;
+
+#define P_BYTES sizeof(Subcode)
+
+unsigned char basis[ HIGH_LIM ];
+unsigned char data_2_basis[ HIGH_LIM ];
+
+void setup_reed_solomon(void)
+{
+  int shiftreg=INIT_VALUE;
+  int i;
+
+  for (i = 1; i < HIGH_LIM; i++) {
+
+    basis[i] = shiftreg;
+    data_2_basis[shiftreg] = i;
+
+    /* this simulates a shift register with feedback */
+    shiftreg =   (((shiftreg << 1) & HIGH_VAL)
+		| ((shiftreg & HIGH_BIT) >> (BITS - 1)))
+                /* the following modifies bit 1, 
+		   when bit HIGH_BIT is high */
+		^ ((shiftreg & HIGH_BIT) >> (BITS - 2));
+
   }
+  basis [0] = basis[HIGH_LIM-1];
+  data_2_basis[0] = 0xff;
+
+#if 0
+  for (i = 0; i <= HIGH_VAL; i++) {
+    printf("%2x: log=%2x, alog=%2x\n", i, data_2_basis[i], basis[i]);
+  }
+#endif
+}
+
+int check_rw(unsigned char *inbuf)
+{
+  int i;
+  static unsigned P_errors = 0;
+  int errors=0;
+  int bsize=4*P_BYTES;
+
+  for (;bsize; bsize -= sizeof(Subcode), inbuf += sizeof(Subcode)) {
+    int p[4] = {0,0,0,0};
+    int q[2] = {0,0};
+
+    /* first stage: check all 24 bytes against parity P */
+    for (i = 0; i < (int)P_BYTES; i++) {
+      unsigned char data;
+      data = inbuf[i] & 0x3f;
+      if (data != 0) {
+	unsigned char base = data_2_basis[data];
+	p[0] ^= data;
+	p[1] ^= basis[(base+   P_BYTES-1-i ) % HIGH_VAL];
+	p[2] ^= basis[(base+2*(P_BYTES-1-i)) % HIGH_VAL];
+	p[3] ^= basis[(base+3*(P_BYTES-1-i)) % HIGH_VAL];
+      }
+    }
+
+    if (p[0] || p[1] || p[2] || p[3]) {
+      errors++;
+fprintf(stderr, "P: %2x %2x %2x %2x\n", p[0], p[1], p[2], p[3]);
+    }
+
+  /* second stage: check first 4 bytes against parity Q */
+#define Q_BYTES 4
+    for (i = 0; i < Q_BYTES; i++) {
+      unsigned char data;
+      data = inbuf[i] & 0x3f;
+      if (data != 0) {
+	q[0] ^= data;
+	q[1] ^= basis[(data_2_basis[data]+Q_BYTES-1-i) % HIGH_VAL];
+      }
+    }
+
+    /* count number of errors */
+    if (q[0] || q[1]) {
+      /* build locator and corrector syndromes */
+      unsigned char where;
+fprintf(stderr, "Q: %2x %2x\n", q[0], q[1]);
+      where = Q_BYTES - 1 - (4 + data_2_basis[q[1]] - data_2_basis[q[0]]) % 4;
+
+#if 0
+printf("Correct at %2d: %2x -> %2x\n",where, inbuf[where], inbuf[where] ^ q[0]);
+#endif
+      /* do the correction */
+      inbuf[where] ^= q[0];
+    }
+    if (errors != 0) {
+      memset(inbuf, 0, P_BYTES);
+    }
+  }
+  return errors != 0;
+}
+
+
+static unsigned char sectbuff[CD_FRAMESIZE_RAW + SUBSIZE];
+
+int main(int argc, char **argv)
+{
+  FILE *fp;
+  unsigned char *subs;
+  unsigned count_sectors = 0;
+  unsigned sectors_trackrel = 0;
+  int with_data = 1;
+
+  if (argc == 3 && !strcmp(argv[1], "-s")) {
+    argv++; argc--;
+    with_data = 0;
+  }
+  if (argc != 2) return 1;
+
+  fp = fopen(argv[1], "rb");
+  if (fp == NULL) return 2;
+
+  setup_reed_solomon();
+  for (;! feof(fp); count_sectors++, sectors_trackrel++ ) {
+
+    if (with_data) {
+      fread(sectbuff, CD_FRAMESIZE_RAW + SUBSIZE, 1, fp);
+      subs = sectbuff + CD_FRAMESIZE_RAW;
+    } else {
+      fread(sectbuff, SUBSIZE, 1, fp);
+      subs = sectbuff;
+    }
+    crc_errors += check_rw(subs);
+    /* x(subs, &sectors_trackrel); */
+  }
+
+
   if (count_sectors && crc_errors)
     fprintf(stdout, "%u sectors with errors of %u sectors total.\n",
             crc_errors, count_sectors);
