@@ -14,10 +14,10 @@ void SpeedSelectSCSISony (unsigned speed);
 void SpeedSelectSCSIPhilipsCDD2600 (unsigned speed);
 void SpeedSelectSCSINEC (unsigned speed);
 void SpeedSelectSCSIToshiba (unsigned speed);
-void ReadCddaMMC12 (unsigned char *p, long lSector, unsigned long ectorBurstVal );
-void ReadCdda12 (unsigned char *p, long lSector, unsigned long SecorBurstVal );
-void ReadCdda10 (unsigned char *p, long lSector, unsigned long SecorBurstVal );
-void ReadStandard (unsigned char *p, long lSector, unsigned long SctorBurstVal );
+void ReadCddaMMC12 (unsigned char *p, long lSector, unsigned long SectorBurstVal, int with_errorblockflags );
+void ReadCdda12 (unsigned char *p, long lSector, unsigned long SectorBurstVal, int with_errorblockflags );
+void ReadCdda10 (unsigned char *p, long lSector, unsigned long SectorBurstVal, int with_errorblockflags );
+void ReadStandard (unsigned char *p, long lSector, unsigned long SectorBurstVal, int with_errorblockflags );
 unsigned ReadTocCdrSCSI ( TOC *toc );
 unsigned ReadTocSCSI ( TOC *toc );
 unsigned ReadFirstSessionTOCSony ( unsigned tracks );
